//
// LDASTools frameCPP - A library implementing the LIGO/Virgo frame
// specification
//
// Copyright (C) 2018 California Institute of Technology
//
// LDASTools frameCPP is free software; you may redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 (GPLv2) of the
// License or at your discretion, any later version.
//
// LDASTools frameCPP is distributed in the hope that it will be useful, but
// without any warranty or even the implied warranty of merchantability
// or fitness for a particular purpose. See the GNU General Public
// License (GPLv2) for more details.
//
// Neither the names of the California Institute of Technology (Caltech),
// The Massachusetts Institute of Technology (M.I.T), The Laser
// Interferometer Gravitational-Wave Observatory (LIGO), nor the names
// of its contributors may be used to endorse or promote products derived
// from this software without specific prior written permission.
//
// You should have received a copy of the licensing terms for this
// software included in the file LICENSE located in the top-level
// directory of this package. If you did not, you can view a copy at
// http://dcc.ligo.org/M1500244/LICENSE
//

#include "framecpp/Common/Description.hh"
#include "framecpp/Common/TOCInfo.hh"

#include "framecpp/Version4/FrameSpec.hh"
#include "framecpp/Version4/FrTOCTable.hh"
#include "framecpp/Version4/FrSE.hh"
#include "framecpp/Version4/FrSH.hh"
#include "framecpp/Version4/FrTOC.hh"

#include "framecpp/Version4/STRING.hh"

using FrameCPP::Common::Description;
using FrameCPP::Common::FrameSpec;
using FrameCPP::Common::TOCInfo;

namespace FrameCPP
{
    namespace Version_4
    {
        //===================================================================
        //===================================================================
        void
        FrTOCTable::Description( Common::Description& Desc )
        {
            Desc( FrSE( "nTable",
                        "INT_4U",
                        "Number of unique FrTable names in file." ) );
            Desc( FrSE( "nameTable", "*STRING", "Array of FrTable names" ) );
            Desc( FrSE( "positionTable",
                        "*INT_8U",
                        "Array of lists of FrTable offset positions, in bytes,"
                        " from beginning of file (size of nFrame*nTable)" ) );
        }

        void
        FrTOCTable::QueryTable( const Common::TOCInfo& Info,
                                INT_4U                 FrameOffset,
                                INT_8U                 Position )
        {
            STRING name;

            Info.TOCQuery( TOCInfo::IC_NAME,
                           TOCInfo::DT_STRING_2,
                           &name,
                           TOCInfo::IC_EOQ );

            table_info_type& i( m_info[ name ] );
            i.resize( FrameOffset + 1 );
            i[ FrameOffset ] = Position;
        }
    } // namespace Version_4
} // namespace FrameCPP
